package com.bergerkiller.bukkit.nolagg.spawnlimit;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.WeakHashMap;

import org.bukkit.Material;
import org.bukkit.World;
import org.bukkit.block.Block;
import org.bukkit.entity.Entity;

import com.bergerkiller.bukkit.nolagg.Util;

public class SpawnHandler {
	public static final GroupLimiter GLOBAL = new GroupLimiter();
	public static final GroupLimiter WORLDDEFAULT = new GroupLimiter();
	private static final Map<String, WorldLimiter> limiters = new HashMap<String, WorldLimiter>();
	public static WorldLimiter getWorldLimiter(World world) {
		return getWorldLimiter(world.getName());
	}
	public static WorldLimiter getWorldLimiter(String worldname) {
		worldname = worldname.toLowerCase();
		WorldLimiter lim = limiters.get(worldname);
		if (lim == null) {
			lim = new WorldLimiter();
			lim.load(WORLDDEFAULT);
			limiters.put(worldname, lim);
		}
		return lim;
	}
	
	public static boolean isAnimal(String name) {
		return Util.in(name, "cow", "pig", "sheep", "chicken", "wolf", "squid");
	}
	public static boolean isMonster(String name) {
		return Util.in(name, "creeper", "skeleton", "zombie", "slime", "skeleton", "pigzombie", "spider", "giant", "ghast", "enderman", "cavespider", "ender_dragon", "snowman");
	}
	public static boolean isItem(String name) {
		return name.toLowerCase().startsWith("item");
	}
	public static boolean isFalling(String name) {
		return name.toLowerCase().startsWith("falling");
	}
	public static boolean canFall(Material type) {
		switch (type) {
		case SAND : return true;
		case GRAVEL : return true;
		default : return false;
		}
	}
	public static EntityLimiter getBlockFallingLimit(Block block) {
		WorldLimiter wlim = getWorldLimiter(block.getWorld());
		Material type = block.getType();
		if (canFall(type)) {
			return wlim.getEntityLimits("falling" + type.toString());
		} else {
			return null;
		}
	}
	
	private static WeakHashMap<Entity, Boolean> ignoredEntities = new WeakHashMap<Entity, Boolean>();
	public static void ignoreSpawn(Entity entity) {
		ignoredEntities.put(entity, true);
	}
	public static boolean isIgnored(Entity entity) {
		return ignoredEntities.containsKey(entity);
	}
	
	public static EntityLimiter getEntityLimits(Entity entity) {
		if (isIgnored(entity)) return new EntityLimiter();
		return getWorldLimiter(entity.getWorld()).getEntityLimits(entity);
	}
	public static void handleDespawn(Entity entity) {
		getEntityLimits(entity).despawn();
		ignoredEntities.remove(entity);
	}
	public static void reset() {
		GLOBAL.reset();
		for (WorldLimiter lim : limiters.values()) lim.reset();
	}
	public static void clear() {
		GLOBAL.clear();
		WORLDDEFAULT.clear();
		limiters.clear();
	}

	public static void update(World world, List<Entity> entities) {
		getWorldLimiter(world).update(entities);
	}

}
